@@ -3,8 +3,8 @@
  * Copyright (c) 2007-2013 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2008-2009 Mellanox Technologies. All rights reserved.
  * Copyright (c) 2009      IBM Corporation.  All rights reserved.
- * Copyright (c) 2011-2013 Los Alamos National Security, LLC.  All
- *                         rights reserved.
+ * Copyright (c) 2011-2014 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  *
  * $COPYRIGHT$
  * 
@@ -59,6 +59,7 @@
 #include "opal/util/show_help.h"
 #include "opal/util/output.h"
 #include "opal/util/error.h"
+#include "opal/util/alfg.h"
 #include "opal_stdint.h"
 
 #include "btl_openib_endpoint.h"
@@ -151,6 +152,9 @@ typedef struct udcm_module {
 
     /* This module's modex message */
     modex_msg_t modex;
+
+    /** The channel is being monitored */
+    bool channel_monitored;
 } udcm_module_t;
 
 /*
@@ -295,6 +299,7 @@ static void *udcm_cq_event_dispatch(int fd, int flags, void *context);
 static void *udcm_message_callback (void *context);
 
 static void udcm_set_message_timeout (udcm_message_sent_t *message);
+static void udcm_cancel_message_timeout (udcm_message_sent_t *message);
 
 static int udcm_module_init (udcm_module_t *m, mca_btl_openib_module_t *btl);
 
@@ -356,7 +361,9 @@ static int udcm_max_retry;
 static int udcm_timeout;
 
 /* seed for rand_r. remove me when opal gets a random number generator */
-static unsigned udcm_random_seed = 0;
+/* Uses the OPAL ALFG RNG */
+static uint32_t udcm_random_seed = 0;
+static opal_rng_buff_t udcm_rand_buff;
 
 static struct timeval udcm_timeout_tv;
 
@@ -398,7 +405,7 @@ static void udcm_component_register(void)
                                            0, 0, OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                            &udcm_timeout);
 
-    udcm_max_retry = 10;
+    udcm_max_retry = 25;
     (void) mca_base_component_var_register(&mca_btl_openib_component.super.btl_version,
                                            "connect_udcm_max_retry", "Maximum number of times "
                                            "to retry sending a udcm connection message",
@@ -456,7 +463,7 @@ static int udcm_component_query(mca_btl_openib_module_t *btl,
 
         /* seed the random number generator */
         udcm_random_seed = time (NULL);
-
+        opal_srand(&udcm_rand_buff,udcm_random_seed);
         /* All done */
         *cpc = (ompi_btl_openib_connect_base_module_t *) m;
         BTL_VERBOSE(("available for use on %s:%d",
@@ -604,6 +611,7 @@ static int udcm_module_init (udcm_module_t *m, mca_btl_openib_module_t *btl)
     /* Monitor the fd associated with the completion channel */
     ompi_btl_openib_fd_monitor(m->cm_channel->fd, OPAL_EV_READ,
                                udcm_cq_event_dispatch, m);
+    m->channel_monitored = true;
 
     OBJ_CONSTRUCT(&m->cm_lock, opal_mutex_t);
     OBJ_CONSTRUCT(&m->cm_send_lock, opal_mutex_t);
@@ -649,14 +657,15 @@ udcm_module_start_connect(ompi_btl_openib_connect_base_module_t *cpc,
 #endif
 
 
+    opal_mutex_lock (&udep->udep_lock);
+
     if (MCA_BTL_IB_CLOSED != lcl_ep->endpoint_state) {
+        opal_mutex_unlock (&udep->udep_lock);
         BTL_VERBOSE(("already ongoing %p. state = %d",
                      (void *) lcl_ep, lcl_ep->endpoint_state));
         return OMPI_SUCCESS;
     }
 
-    opal_mutex_lock (&udep->udep_lock);
-
     do {
         opal_atomic_wmb ();
 
@@ -702,11 +711,13 @@ static int udcm_module_finalize(mca_btl_openib_module_t *btl,
 
     m->cm_exiting = true;
 
-    /* stop monitoring the channel's fd before destroying the listen qp */
-    ompi_btl_openib_fd_unmonitor(m->cm_channel->fd, udcm_unmonitor, (void *)&barrier);
+    if (m->channel_monitored) {
+        /* stop monitoring the channel's fd before destroying the listen qp */
+        ompi_btl_openib_fd_unmonitor(m->cm_channel->fd, udcm_unmonitor, (void *)&barrier);
 
-    while (0 == barrier) {
-        sched_yield();
+        while (0 == barrier) {
+            sched_yield();
+        }
     }
 
     opal_mutex_lock (&m->cm_lock);
@@ -848,26 +859,31 @@ static void udcm_module_destroy_listen_qp (udcm_module_t *m)
 {
     struct ibv_qp_attr attr;
     struct ibv_wc wc;
-    mca_btl_openib_async_cmd_t async_command;
 
     if (NULL == m->listen_qp) {
         return;
     }
 
-    /* Tell the openib async thread to ignore ERR state on the QP
-       we are about to manually set the ERR state on */
-    async_command.a_cmd = OPENIB_ASYNC_IGNORE_QP_ERR;
-    async_command.qp = m->listen_qp;
-    if (write(mca_btl_openib_component.async_pipe[1],
-              &async_command, sizeof(mca_btl_openib_async_cmd_t))<0){
-        BTL_ERROR(("Failed to write to pipe [%d]",errno));
-        return;
-    }
-    /* wait for ok from thread */
-    if (OMPI_SUCCESS !=
-        btl_openib_async_command_done(OPENIB_ASYNC_IGNORE_QP_ERR)) {
-        BTL_ERROR(("Command to openib async thread to ignore QP ERR state failed"));
+#if OPAL_HAVE_THREADS
+    if (mca_btl_openib_component.use_async_event_thread &&
+        -1 != mca_btl_openib_component.async_pipe[1]) {
+        /* Tell the openib async thread to ignore ERR state on the QP
+           we are about to manually set the ERR state on */
+        mca_btl_openib_async_cmd_t async_command;
+        async_command.a_cmd = OPENIB_ASYNC_IGNORE_QP_ERR;
+        async_command.qp = m->listen_qp;
+        if (write(mca_btl_openib_component.async_pipe[1],
+                  &async_command, sizeof(mca_btl_openib_async_cmd_t))<0){
+            BTL_ERROR(("Failed to write to pipe [%d]",errno));
+            return;
+        }
+        /* wait for ok from thread */
+        if (OMPI_SUCCESS !=
+                        btl_openib_async_command_done(OPENIB_ASYNC_IGNORE_QP_ERR)) {
+            BTL_ERROR(("Command to openib async thread to ignore QP ERR state failed"));
+        }
     }
+#endif
 
     do {
         /* Move listen QP into the ERR state to cancel all outstanding
@@ -1028,11 +1044,10 @@ static uint32_t max_inline_size(int qp, mca_btl_openib_device_t *device)
     return 0;
 }
 
-/* rand_r is a weak random number generator but should be suitable for our
- * purposes */
+/* Using OPAL's Additive Lagged Fibonacci RNG */
 static inline uint32_t udcm_random (void)
 {
-    return (uint32_t) rand_r (&udcm_random_seed);
+    return opal_rand(&udcm_rand_buff);
 }
 
 /* mark: rc helper functions */
@@ -1512,11 +1527,13 @@ static int udcm_new_message (mca_btl_base_endpoint_t *lcl_ep,
 
     message->endpoint = lcl_ep;
 
+    udcm_set_message_timeout (message);
+
     opal_atomic_wmb ();
 
     *msgp = message;
 
-    BTL_VERBOSE(("created message with type %d", type));
+    BTL_VERBOSE(("created message %p with type %d", (void *) message, type));
 
     return OMPI_SUCCESS;
 }
@@ -1554,11 +1571,12 @@ static int udcm_send_request (mca_btl_base_endpoint_t *lcl_ep,
 
     if (0 != (rc = udcm_post_send (lcl_ep, msg->data, m->msg_length, 0))) {
         BTL_VERBOSE(("error posting REQ"));
+
+        udcm_cancel_message_timeout (msg);
+
         return rc;
     }
 
-    udcm_set_message_timeout (msg);
-
     return 0;
 }
 
@@ -1576,11 +1594,12 @@ static int udcm_send_complete (mca_btl_base_endpoint_t *lcl_ep,
     rc = udcm_post_send (lcl_ep, msg->data, sizeof (udcm_msg_hdr_t), 0);
     if (0 != rc) {
         BTL_VERBOSE(("error posting complete"));
+
+        udcm_cancel_message_timeout (msg);
+
         return rc;
     }
 
-    udcm_set_message_timeout (msg);
-
     return 0;
 }
 
@@ -1601,11 +1620,12 @@ static int udcm_send_reject (mca_btl_base_endpoint_t *lcl_ep,
     rc = udcm_post_send (lcl_ep, msg->data, sizeof (udcm_msg_hdr_t), 0);
     if (0 != rc) {
         BTL_VERBOSE(("error posting rejection"));
+
+        udcm_cancel_message_timeout (msg);
+
         return rc;
     }
 
-    udcm_set_message_timeout (msg);
-
     return 0;
 }
 
@@ -1625,6 +1645,7 @@ static int udcm_handle_ack (udcm_module_t *m, const uintptr_t ctx, const uint16_
                             const uint32_t rem_qp)
 {
     udcm_message_sent_t *msg, *next;
+    bool found = false;
 
     opal_mutex_lock (&m->cm_timeout_lock);
 
@@ -1638,6 +1659,7 @@ static int udcm_handle_ack (udcm_module_t *m, const uintptr_t ctx, const uint16_
         }
 
         BTL_VERBOSE(("found matching message"));
+        found = true;
 
         /* found it */
         opal_list_remove_item (&m->flying_messages, &msg->super);
@@ -1646,6 +1668,10 @@ static int udcm_handle_ack (udcm_module_t *m, const uintptr_t ctx, const uint16_
         break;
     }
 
+    if (!found) {
+        BTL_VERBOSE(("message %p not found in the list of flying messages", (void *) ctx));
+    }
+
     opal_mutex_unlock (&m->cm_timeout_lock);
 
     return OMPI_SUCCESS;
@@ -1796,7 +1822,7 @@ static int udcm_process_messages (struct ibv_cq *event_cq, udcm_module_t *m)
         msg_num = (int)(wc[i].wr_id & (~UDCM_WR_DIR_MASK));
 
         if (IBV_WC_SUCCESS != wc[i].status) {
-            BTL_VERBOSE(("recv work request for buffer %d failed, code = %d",
+            BTL_ERROR(("recv work request for buffer %d failed, code = %d",
                          msg_num, wc[i].status));
             count = -1;
             break;
@@ -2109,14 +2135,14 @@ static void udcm_send_timeout (evutil_socket_t fd, short event, void *arg)
 
         msg->tries++;
 
+        udcm_set_message_timeout (msg);
+
         if (0 != udcm_post_send (lcl_ep, msg->data, msg->length, 0)) {
             BTL_VERBOSE(("error reposting message"));
             ompi_btl_openib_fd_run_in_main(mca_btl_openib_endpoint_invoke_error,
                                            lcl_ep);
             break;
         }
-
-        udcm_set_message_timeout (msg);
     } while (0);
 }
 
@@ -2124,6 +2150,8 @@ static void udcm_set_message_timeout (udcm_message_sent_t *message)
 {
     udcm_module_t *m = UDCM_ENDPOINT_MODULE(message->endpoint);
 
+    BTL_VERBOSE(("activating timeout for message %p", (void *) message));
+
     opal_mutex_lock (&m->cm_timeout_lock);
 
     opal_list_append (&m->flying_messages, &message->super);
@@ -2135,6 +2163,23 @@ static void udcm_set_message_timeout (udcm_message_sent_t *message)
     opal_mutex_unlock (&m->cm_timeout_lock);
 }
 
+static void udcm_cancel_message_timeout (udcm_message_sent_t *message)
+{
+    udcm_module_t *m = UDCM_ENDPOINT_MODULE(message->endpoint);
+
+    BTL_VERBOSE(("cancelling timeout for message %p", (void *) message));
+
+    opal_mutex_lock (&m->cm_timeout_lock);
+
+    opal_list_remove_item (&m->flying_messages, &message->super);
+
+    /* start the event */
+    opal_event_evtimer_del (&message->event);
+    message->event_active = false;
+
+    opal_mutex_unlock (&m->cm_timeout_lock);
+}
+
 /* mark: xrc connection support */
 
 /* XRC support functions */
@@ -2547,11 +2592,12 @@ static int udcm_xrc_send_request (mca_btl_base_endpoint_t *lcl_ep, mca_btl_base_
 
     if (0 != (rc = udcm_post_send (lcl_ep, msg->data, sizeof (udcm_msg_hdr_t), 0))) {
         BTL_VERBOSE(("error posting XREQ"));
+
+        udcm_cancel_message_timeout (msg);
+
         return rc;
     }
 
-    udcm_set_message_timeout (msg);
-
     return 0;
 }
 
@@ -2584,11 +2630,12 @@ static int udcm_xrc_send_xresponse (mca_btl_base_endpoint_t *lcl_ep, mca_btl_bas
     rc = udcm_post_send (lcl_ep, msg->data, m->msg_length, 0);
     if (0 != rc) {
         BTL_VERBOSE(("error posting complete"));
+
+        udcm_cancel_message_timeout (msg);
+
         return rc;
     }
 
-    udcm_set_message_timeout (msg);
-
     return 0;
 }
 
