@@ -13,7 +13,7 @@
  *                         reserved.
  * Copyright (c) 2007      The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013-2014 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -53,6 +53,7 @@ static void endpoint_construct(mca_btl_base_endpoint_t* endpoint)
     endpoint->endpoint_proc = NULL;
     endpoint->endpoint_proc_index = -1;
     endpoint->endpoint_exiting = false;
+    endpoint->endpoint_connectivity_checked = false;
 
     for (i=0; i<USNIC_NUM_CHANNELS; ++i) {
         endpoint->endpoint_remote_addr.qp_num[i] = 0;
@@ -122,8 +123,10 @@ static void endpoint_destruct(mca_btl_base_endpoint_t* endpoint)
 
     /* release owning proc */
     proc = endpoint->endpoint_proc;
-    proc->proc_endpoints[endpoint->endpoint_proc_index] = NULL;
-    OBJ_RELEASE(proc);
+    if (NULL != proc) {
+        proc->proc_endpoints[endpoint->endpoint_proc_index] = NULL;
+        OBJ_RELEASE(proc);
+    }
 
     free(endpoint->endpoint_rx_frag_info);
 
