@@ -11,7 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2006      Sandia National Laboratories. All rights
  *                         reserved.
- * Copyright (c) 2009-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2014 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,6 +25,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <time.h>
 
 #include "opal/class/opal_bitmap.h"
 #include "opal/prefetch.h"
@@ -33,7 +34,6 @@
 #include "opal/include/opal_stdint.h"
 #include "opal/util/show_help.h"
 
-#include "ompi/mca/rte/rte.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/btl_base_error.h"
 #include "ompi/mca/mpool/base/base.h"
@@ -41,6 +41,7 @@
 #include "ompi/memchecker.h"
 
 #include "btl_usnic.h"
+#include "btl_usnic_connectivity.h"
 #include "btl_usnic_frag.h"
 #include "btl_usnic_proc.h"
 #include "btl_usnic_endpoint.h"
@@ -56,6 +57,7 @@ ompi_btl_usnic_channel_finalize(
     ompi_btl_usnic_module_t *module,
     struct ompi_btl_usnic_channel_t *channel);
 
+
 /* Compute and set the proper value for sfrag->sf_size.  This must not be used
  * during usnic_alloc, since the PML might change the segment size after
  * usnic_alloc returns. */
@@ -76,6 +78,81 @@ static inline void compute_sf_size(ompi_btl_usnic_send_frag_t *sfrag)
     sfrag->sf_size += frag->uf_src_seg[1].seg_len;
 }
 
+/* The call to ibv_create_ah() may initiate an ARP resolution, and may
+ * therefore take some time to complete.  Hence, it will return 1 of 3
+ * things:
+ *
+ * 1. a valid new ah
+ * 2. NULL and errno == EAGAIN (ARP not complete; try again later)
+ * 3. NULL and errno != EAGAIN (fatal error)
+ *
+ * Since ibv_create_ah() is therefore effectively non-blocking, we
+ * gang all the endpoint ah creations here in this loop so that we can
+ * get some parallelization of ARP resolution.
+ */
+static int create_ahs(size_t array_len, size_t num_endpoints,
+                      struct mca_btl_base_endpoint_t** endpoints,
+                      ompi_btl_usnic_module_t *module)
+{
+    size_t i;
+    struct ibv_ah_attr ah_attr;
+    size_t num_ah_created, last_num_ah_created;
+    time_t ts_last_created;
+
+    /* memset the ah_attr to both silence valgrind warnings (since the
+       attr struct ends up getting written down an fd to the kernel)
+       and actually zero out all the fields that we don't care about /
+       want to be logically false. */
+    memset(&ah_attr, 0, sizeof(ah_attr));
+    ah_attr.is_global = 1;
+    ah_attr.port_num = 1;
+
+    ts_last_created = time(NULL);
+    last_num_ah_created = num_ah_created = 0;
+    while (num_ah_created < num_endpoints) {
+        for (i = 0; i < array_len; i++) {
+            if (NULL != endpoints[i] &&
+                NULL == endpoints[i]->endpoint_remote_ah) {
+                ah_attr.grh.dgid = endpoints[i]->endpoint_remote_addr.gid;
+                endpoints[i]->endpoint_remote_ah =
+                    ibv_create_ah(module->pd, &ah_attr);
+                if (NULL != endpoints[i]->endpoint_remote_ah) {
+                    ts_last_created = time(NULL);
+                    ++num_ah_created;
+                } else if (EAGAIN != errno) {
+                    opal_show_help("help-mpi-btl-usnic.txt", "ibv API failed",
+                                   true,
+                                   ompi_process_info.nodename,
+                                   ibv_get_device_name(module->device),
+                                   module->port_num,
+                                   "ibv_create_ah()", __FILE__, __LINE__,
+                                   "Failed to create an address handle");
+                    return OMPI_ERR_OUT_OF_RESOURCE;
+                }
+            }
+        }
+
+        /* Has it been too long since our last AH creation (ARP
+           resolution)?  If so, we're probably never going to finish,
+           so just bail. */
+        if (num_ah_created < num_endpoints &&
+            time(NULL) > (ts_last_created + mca_btl_usnic_component.arp_timeout)) {
+            opal_show_help("help-mpi-btl-usnic.txt", "ibv_create_ah timeout",
+                           true,
+                           ompi_process_info.nodename,
+                           ibv_get_device_name(module->device),
+                           module->port_num,
+                           module->if_name,
+                           mca_btl_usnic_component.arp_timeout);
+            return OMPI_ERR_UNREACH;
+        }
+    }
+
+    opal_output_verbose(5, USNIC_OUT,
+                        "btl:usnic: made %" PRIsize_t " address handles", num_endpoints);
+    return OMPI_SUCCESS;
+}
+
 /*
  *  Add procs to this BTL module, receiving endpoint information from
  *  the modex.
@@ -103,6 +180,8 @@ static int usnic_add_procs(struct mca_btl_base_module_t* base_module,
         ompi_btl_usnic_proc_t* usnic_proc;
         mca_btl_base_endpoint_t* usnic_endpoint;
 
+        endpoints[i] = NULL;
+
         /* Do not create loopback usnic connections */
         if (ompi_proc == my_proc) {
             continue;
@@ -129,11 +208,14 @@ static int usnic_add_procs(struct mca_btl_base_module_t* base_module,
         rc = ompi_btl_usnic_create_endpoint(module, usnic_proc,
                 &usnic_endpoint);
         if (OMPI_SUCCESS != rc) {
+            opal_output_verbose(5, USNIC_OUT,
+                                "btl:usnic:%s: unable to create endpoint for module=%p proc=%p\n",
+                                __func__, (void *)module, (void *)usnic_proc);
             OBJ_RELEASE(usnic_proc);
             continue;
         }
 
-        /* Add to array of all procs */
+        /* Add to array of all procs (proc_match gave us a reference) */
         opal_pointer_array_add(&module->all_procs, usnic_proc);
 
         opal_output_verbose(5, USNIC_OUT,
@@ -150,6 +232,24 @@ static int usnic_add_procs(struct mca_btl_base_module_t* base_module,
     opal_output_verbose(5, USNIC_OUT,
                         "btl:usnic: made %" PRIsize_t " endpoints", count);
 
+    /* Create address handles for all the newly-created endpoints */
+    rc = create_ahs(nprocs, count, endpoints, module);
+    if (OMPI_SUCCESS != rc) {
+        for (i = 0; i < nprocs; i++) {
+            if (NULL != endpoints[i]) {
+                opal_list_remove_item(&(module->all_endpoints),
+                                      &(endpoints[i]->endpoint_endpoint_li));
+                OBJ_RELEASE(endpoints[i]);
+                endpoints[i] = NULL;
+            }
+            opal_bitmap_clear_bit(reachable, i);
+        }
+        /* Allow falling through to return OMPI_SUCCESS, even though
+           we couldn't reach everyone (and therefore we gave up).  Let
+           the PML know that this module basically isn't reachable,
+           but it's free to continue with other BTL modules. */
+    }
+
     return OMPI_SUCCESS;
 }
 
@@ -199,6 +299,7 @@ static int usnic_del_procs(struct mca_btl_base_module_t *base_module,
             for (index = 0; index < module->all_procs.size; ++index) {
                 if (opal_pointer_array_get_item(&module->all_procs, index) ==
                         proc) {
+                    OBJ_RELEASE(proc);
                     opal_pointer_array_set_item(&module->all_procs, index,
                             NULL);
                     break;
@@ -365,7 +466,6 @@ prepare_src_small(
     ompi_btl_usnic_send_frag_t *frag;
     ompi_btl_usnic_small_send_frag_t *sfrag;
     size_t payload_len;
-    int rc;
 
     payload_len = *size + reserve;
     assert(payload_len <= module->max_frag_payload); /* precondition */
@@ -399,7 +499,7 @@ prepare_src_small(
     if (OPAL_UNLIKELY(opal_convertor_need_buffers(convertor))) {
         /* put user data just after end of 1st seg (upper layer header) */
         assert(payload_len <= module->max_frag_payload);
-        rc = usnic_convertor_pack_simple(
+        usnic_convertor_pack_simple(
                 convertor,
                 (IOVBASE_TYPE*)(intptr_t)(frag->sf_base.uf_src_seg[0].seg_addr.lval + reserve),
                 *size,
@@ -439,14 +539,11 @@ pack_chunk_seg_from_frag(
     size_t copylen;
     size_t seg_space;
     size_t max_data;
-    mca_btl_base_descriptor_t *desc;
 
     assert(NULL != lfrag);
     /* never should be attempting to pack if we've already packed everything */
     assert(lfrag->lsf_pack_bytes_left > 0);
 
-    desc = &lfrag->lsf_base.sf_base.uf_base;
-
     seg = ompi_btl_usnic_chunk_segment_alloc(module);
     if (OPAL_UNLIKELY(NULL == seg)) {
         /* TODO look at ways to deal with this case more gracefully, possibly as
@@ -721,6 +818,20 @@ usnic_prepare_src(
     size_t osize = *size;
 #endif
 
+    /* Do we need to check the connectivity? */
+    if (mca_btl_usnic_component.connectivity_enabled &&
+        OPAL_UNLIKELY(!endpoint->endpoint_connectivity_checked)) {
+        ompi_btl_usnic_connectivity_ping(module->local_addr.ipv4_addr,
+                                         module->local_addr.connectivity_udp_port,
+                                         endpoint->endpoint_remote_addr.ipv4_addr,
+                                         endpoint->endpoint_remote_addr.cidrmask,
+                                         endpoint->endpoint_remote_addr.connectivity_udp_port,
+                                         endpoint->endpoint_remote_addr.mac,
+                                         endpoint->endpoint_proc->proc_ompi->proc_hostname,
+                                         endpoint->endpoint_remote_addr.mtu);
+        endpoint->endpoint_connectivity_checked = true;
+    }
+
     /*
      * if total payload len fits in one MTU use small send, else large
      */
@@ -971,7 +1082,7 @@ usnic_do_resends(
         endpoint = sseg->ss_parent_frag->sf_endpoint;
 
         /* clobber any stale piggy-backed ACK */
-        sseg->ss_base.us_btl_header->ack_seq = 0;
+        sseg->ss_base.us_btl_header->ack_present = 0;
 
         /* Only post this segment if not already posted */
         if (sseg->ss_send_posted == 0) {
@@ -1019,11 +1130,8 @@ usnic_handle_large_send(
     ompi_btl_usnic_large_send_frag_t *lfrag;
     ompi_btl_usnic_btl_chunk_header_t *chp;
     ompi_btl_usnic_send_segment_t *sseg;
-    mca_btl_base_descriptor_t *desc;
     size_t payload_len;
 
-    desc = &frag->sf_base.uf_base;
-
     assert(frag->sf_base.uf_type == OMPI_BTL_USNIC_FRAG_LARGE_SEND);
     lfrag = (ompi_btl_usnic_large_send_frag_t *)frag;
     if (lfrag->lsf_cur_offset == 0) {
@@ -1493,7 +1601,7 @@ static void module_async_event_callback(int fd, short flags, void *arg)
 
         case IBV_EVENT_QP_FATAL:
         case IBV_EVENT_PORT_ERR:
-#if BTL_USNIC_HAVE_IBV_EVENT_GID_CHANGE
+#if HAVE_DECL_IBV_EVENT_GID_CHANGE
         case IBV_EVENT_GID_CHANGE:
 #endif
         default:
@@ -1822,8 +1930,8 @@ get_initial_seq_no(void)
 {
     ompi_btl_usnic_seq_t isn;
     /* only utilize the bottom 62 bits to avoid hitting seq # overflow */
-    isn = (((uint64_t)random() & ((1LL<<30)-1)) << 32) |
-        ((uint64_t)random() & ((1LL<<32)-1));
+    isn = (((uint64_t)opal_rand(&ompi_btl_usnic_rand_buff) & ((1LL<<30)-1)) << 32) |
+        ((uint64_t)opal_rand(&ompi_btl_usnic_rand_buff) & ((1LL<<32)-1));
     isn += 2;      /* guarantee > 1 */
 
     return isn;
@@ -1856,6 +1964,19 @@ int ompi_btl_usnic_module_init(ompi_btl_usnic_module_t *module)
                         "btl:usnic: not sorting devices by NUMA distance (topology support not included)");
 #endif
 
+    /* Setup a connectivity listener */
+    if (mca_btl_usnic_component.connectivity_enabled) {
+        rc = ompi_btl_usnic_connectivity_listen(module);
+        if (OMPI_SUCCESS != rc) {
+            OMPI_ERROR_LOG(rc);
+            ABORT("Failed to notify connectivity agent to listen");
+        }
+    } else {
+        /* If we're not doing a connectivity check, just set the port
+           to 0 */
+        module->local_addr.connectivity_udp_port = 0;
+    }
+
     /* Setup the pointer array for the procs that will be used by this
        module */
     OBJ_CONSTRUCT(&module->all_procs, opal_pointer_array_t);
