@@ -1,6 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2009-2012 Oak Ridge National Laboratory.  All rights reserved.
  * Copyright (c) 2009-2012 Mellanox Technologies.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -90,40 +93,14 @@ enum {
     ML_NUM_OF_FUNCTIONS
 };
 
-struct mca_bcol_base_module_t;
-/* function description */
-struct coll_ml_function_t {
-    int fn_idx;
-    /* module */
-    struct mca_bcol_base_module_t *bcol_module;
-
-    /*
-     *  The following two parameters are used for bcol modules
-     *  that want to do some optimizations based on the fact that
-     *  n functions from the same bcol module are called in a row.
-     *  For example, in the iboffload case, on the first call one
-     *  will want to initialize the MWR, and start to instantiate
-     *  it, but only post it at the end of the last call.
-     *  The index of this function in a sequence of consecutive
-     *  functions from the same bcol
-     */
-    int index_in_consecutive_same_bcol_calls;
-
-    /* number of times functions from this bcol are
-     * called in order
-     */
-    int n_of_this_type_in_a_row;
-
-    /*
-     * number of times functions from this module are called in the
-     * collective operation.
-     */
-    int n_of_this_type_in_collective;
-    int index_of_this_type_in_collective;
+/* ML broadcast algorithms */
+enum {
+    COLL_ML_STATIC_BCAST,
+    COLL_ML_SEQ_BCAST,
+    COLL_ML_UNKNOWN_BCAST,
 };
-typedef struct coll_ml_function_t coll_ml_function_t;
-
 
+struct mca_bcol_base_module_t;
 
 /* collective function arguments - gives
  * one function signature for calling all collective setup
@@ -195,7 +172,7 @@ struct coll_ml_collective_description_t {
     mpi_coll_algorithm_params_t alg_params;
 
     /* list of functions */
-    coll_ml_function_t *functions;
+    mca_bcol_base_function_t *functions;
 
     /* function names - for debugging */
     char **function_names;
@@ -210,8 +187,6 @@ typedef struct coll_ml_collective_description_t coll_ml_collective_description_t
 struct rank_properties_t {
     int rank;
     int leaf;
-    int n_connected_subgroups;
-    int *list_connected_subgroups;
     int num_of_ranks_represented;
 }; typedef struct rank_properties_t rank_properties_t;
 
@@ -237,17 +212,11 @@ struct sub_group_params_t {
 
     /*
      * level in the hierarchy - subgroups at the same
-     * level don't
-     * overlap.
+     * level don't overlap. May not be the same as the
+     * sbgp level.
      */
     int level_in_hierarchy;
 
-    /*
-     * Connected nodes
-     */
-    int n_connected_nodes;
-    int *list_connected_nodes;
-
     /*
      * Information on the ranks in the subgroup.  This includes
      * the rank, and wether or not the rank is a source/sink of
@@ -255,11 +224,6 @@ struct sub_group_params_t {
      */
     rank_properties_t *rank_data;
 
-    /*
-     * Temp list of ranks
-     */
-    int *list_ranks;
-
     /* level one index - for example,
        for( i = 0; i < level_one_index; i++) will loop
        through all level one subgroups, this is significant
@@ -267,7 +231,6 @@ struct sub_group_params_t {
        i.e. all ranks appear once and only once at level one
      */
     int level_one_index;
-
 };
 typedef struct sub_group_params_t sub_group_params_t;
 
@@ -397,6 +360,7 @@ do {                                                                \
     /* pasha - why we duplicate it ? */                             \
     (coll_op)->variable_fn_params.src_desc = desc;                  \
     (coll_op)->variable_fn_params.hier_factor = 1;                  \
+    (coll_op)->variable_fn_params.need_dt_support = false;          \
 } while (0)
 
 /*Full message descriptor*/
@@ -486,9 +450,6 @@ struct mca_coll_ml_component_t {
     /** MCA parameter: Priority of this component */
     int ml_priority;
 
-    /** MCA parameter: Number of levels */
-    int ml_n_levels;
-
     /** MCA parameter: subgrouping components to use */
     char *subgroups_string;
 
@@ -519,17 +480,14 @@ struct mca_coll_ml_component_t {
 
     int use_knomial_allreduce;
 
-    /* Use global knowledge bcast algorithm */
-    bool use_static_bcast;
-
     /* use hdl_framework */
     bool use_hdl_bcast;
 
     /* Enable / Disable fragmentation (0 - off, 1 - on, 2 - auto) */
     int enable_fragmentation;
 
-    /* Use sequential bcast algorithm */
-    bool use_sequential_bcast;
+    /* Broadcast algorithm */
+    int bcast_algorithm;
 
     /* frag size that is used by list memory_manager */
     size_t lmngr_block_size;
@@ -584,6 +542,9 @@ struct mca_coll_ml_component_t {
     /* Temporary hack for IMB test - not all bcols have alltoall */
     bool disable_alltoall;
 
+    /* Disable Reduce */
+    bool disable_reduce;
+
     /* Brucks alltoall mca and other params */
     int use_brucks_smsg_alltoall;
 
@@ -605,12 +566,6 @@ typedef struct mca_coll_ml_component_t mca_coll_ml_component_t;
  */
 OMPI_MODULE_DECLSPEC extern mca_coll_ml_component_t mca_coll_ml_component;
 
-struct mca_coll_ml_route_info_t {
-    int level;
-    int rank;
-};
-typedef struct mca_coll_ml_route_info_t mca_coll_ml_route_info_t;
-
 struct mca_coll_ml_leader_offset_info_t {
     size_t offset;
     int level_one_index;
@@ -629,7 +584,7 @@ struct mca_coll_ml_topology_t {
     int n_levels;
     /* bcols bits that describe supported features/modes */
     uint64_t all_bcols_mode;
-    mca_coll_ml_route_info_t *route_vector;
+    mca_bcol_base_route_info_t *route_vector;
     coll_ml_collective_description_t *hierarchical_algorithms[BCOL_NUM_OF_FUNCTIONS];
     sub_group_params_t *array_of_all_subgroups;
     /* (sbgp, bcol) pairs */
@@ -697,7 +652,7 @@ struct mca_coll_ml_module_t {
     ompi_free_list_t fragment_descriptors;
 
     /** pointer to the payload memory block **/
-    struct ml_memory_block_desc_t *payload_block;
+    struct mca_bcol_base_memory_block_desc_t *payload_block;
 
     /** the maximum size of collective function description */
     int max_dag_size;
@@ -727,6 +682,11 @@ struct mca_coll_ml_module_t {
     mca_coll_ml_collective_operation_description_t *
         coll_ml_allreduce_functions[ML_NUM_ALLREDUCE_FUNCTIONS];
 
+    /** Reduce functions */
+    mca_coll_ml_collective_operation_description_t *
+        coll_ml_reduce_functions[ML_NUM_REDUCE_FUNCTIONS];
+
+
     /** scatter */
     mca_coll_ml_collective_operation_description_t *
         coll_ml_scatter_functions[ML_NUM_SCATTER_FUNCTIONS];
@@ -764,9 +724,6 @@ struct mca_coll_ml_module_t {
     uint64_t fragment_size;
     uint32_t ml_fragment_size;
 
-    /* For carto graph */
-    /* opal_carto_graph_t *sm_graph; */
-    /* opal_carto_graph_t *ib_graph; */
     /* Bcast index table. Pasha: Do we need to define something more generic ?
      the table  x 2 (large/small)*/
     int bcast_fn_index_table[2];
@@ -784,6 +741,9 @@ struct mca_coll_ml_module_t {
     /* On this list we keep coll_op descriptors that were not
      * be able to start, since no ml buffers were available */
     opal_list_t waiting_for_memory_list;
+
+    /* fallback collectives */
+    mca_coll_base_comm_coll_t fallback;
 };
 
 typedef struct mca_coll_ml_module_t mca_coll_ml_module_t;
@@ -812,25 +772,46 @@ int mca_coll_ml_ibarrier_intra(struct ompi_communicator_t *comm,
                                ompi_request_t **req,
                                mca_coll_base_module_t *module);
 
+/* Allreduce with EXTRA TOPO using - blocking */
 int mca_coll_ml_allreduce_dispatch(void *sbuf, void *rbuf, int count,
                                 struct ompi_datatype_t *dtype, struct ompi_op_t *op,
                                 struct ompi_communicator_t *comm, mca_coll_base_module_t *module);
 
+/* Allreduce with EXTRA TOPO using - Non-blocking */
+int mca_coll_ml_allreduce_dispatch_nb(void *sbuf, void *rbuf, int count,
+                                   ompi_datatype_t *dtype, ompi_op_t *op,
+                                   ompi_communicator_t *comm,
+                                   ompi_request_t **req,
+                                   mca_coll_base_module_t *module);
+
 /* Allreduce - blocking */
-int mca_coll_ml_allreduce_intra(void *sbuf, void *rbuf, int count,
+int mca_coll_ml_allreduce(void *sbuf, void *rbuf, int count,
                                 struct ompi_datatype_t *dtype, struct ompi_op_t *op,
                                 struct ompi_communicator_t *comm,
                                 mca_coll_base_module_t *module);
 
-int mca_coll_ml_memsync_intra(mca_coll_ml_module_t *module, int bank_index);
+/* Allreduce - Non-blocking */
+int mca_coll_ml_allreduce_nb(void *sbuf, void *rbuf, int count,
+                                struct ompi_datatype_t *dtype, struct ompi_op_t *op,
+                                struct ompi_communicator_t *comm,
+                                ompi_request_t **req,
+                                mca_coll_base_module_t *module);
 
-/* Reduce blocking */
+/* Reduce - Blocking */
 int mca_coll_ml_reduce(void *sbuf, void *rbuf, int count,
         struct ompi_datatype_t *dtype, struct ompi_op_t *op,
-        int root,
-        struct ompi_communicator_t *comm,
+        int root, struct ompi_communicator_t *comm,
         mca_coll_base_module_t *module);
 
+int mca_coll_ml_reduce_nb(void *sbuf, void *rbuf, int count,
+        struct ompi_datatype_t *dtype, struct ompi_op_t *op,
+        int root, struct ompi_communicator_t *comm,
+        ompi_request_t **req,
+        mca_coll_base_module_t *module);
+
+int mca_coll_ml_memsync_intra(mca_coll_ml_module_t *module, int bank_index);
+
+
 int coll_ml_progress_individual_message(mca_coll_ml_fragment_t *frag_descriptor);
 
 /*
@@ -902,6 +883,17 @@ int mca_coll_ml_fulltree_iboffload_only_hierarchy_discovery(mca_coll_ml_module_t
 
 void mca_coll_ml_allreduce_matrix_init(mca_coll_ml_module_t *ml_module,
                      const mca_bcol_base_component_2_0_0_t *bcol_component);
+static inline int mca_coll_ml_err(const char* fmt, ...)
+{
+    va_list list;
+    int ret;
+
+    va_start(list, fmt);
+    ret = vfprintf(stderr, fmt, list);
+    va_end(list);
+    return ret;
+}
+
 
 #define ML_ERROR(args)                                       \
 do {                                                     \
