@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2012 Inria.  All rights reserved.
  * Copyright (c) 2011      Universite Bordeaux 1
  * Copyright (c) 2013-2014 Los Alamos National Security, LLC. All rights
@@ -71,6 +71,7 @@
 #define ompi_comm_free_keyval_f pompi_comm_free_keyval_f
 #define ompi_comm_free_f pompi_comm_free_f
 #define ompi_comm_get_attr_f pompi_comm_get_attr_f
+#define ompi_comm_get_info_f pompi_comm_get_info_f
 #define ompi_comm_get_errhandler_f pompi_comm_get_errhandler_f
 #define ompi_comm_get_name_f pompi_comm_get_name_f
 #define ompi_comm_get_parent_f pompi_comm_get_parent_f
@@ -80,6 +81,7 @@
 #define ompi_comm_remote_group_f pompi_comm_remote_group_f
 #define ompi_comm_remote_size_f pompi_comm_remote_size_f
 #define ompi_comm_set_attr_f pompi_comm_set_attr_f
+#define ompi_comm_set_info_f pompi_comm_set_info_f
 #define ompi_comm_set_errhandler_f pompi_comm_set_errhandler_f
 #define ompi_comm_set_name_f pompi_comm_set_name_f
 #define ompi_comm_size_f pompi_comm_size_f
