@@ -693,20 +693,14 @@ JNIEXPORT jint JNICALL Java_mpi_Comm_packSize(
     return size;
 }
 
-JNIEXPORT jboolean JNICALL Java_mpi_Comm_iProbe(
-        JNIEnv *env, jobject jthis, jlong comm,
-        jint source, jint tag, jlongArray jStatus)
+JNIEXPORT jlongArray JNICALL Java_mpi_Comm_iProbe(
+        JNIEnv *env, jobject jthis, jlong comm, jint source, jint tag)
 {
     int flag;
     MPI_Status status;
     int rc = MPI_Iprobe(source, tag, (MPI_Comm)comm, &flag, &status);
     ompi_java_exceptionCheck(env, rc);
-    
-    if(flag == 0)
-        return JNI_FALSE;
-
-    ompi_java_status_set(env, jStatus, &status);
-    return JNI_TRUE;
+    return !flag ? NULL : ompi_java_status_new(env, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_Comm_probe(
