@@ -177,33 +177,34 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_createDistGraphAdjacent(
 
 JNIEXPORT void JNICALL Java_mpi_Intracomm_scan(
         JNIEnv *env, jobject jthis, jlong comm,
-        jobject sendBuf, jint sendOff, jobject recvBuf, jint recvOff,
-        jint count, jlong type, jint baseType, jobject op)
+        jobject sendBuf, jboolean sdb, jint sendOff,
+        jobject recvBuf, jboolean rdb, jint recvOff, jint count,
+        jlong type, jint baseType, jobject jOp, jlong hOp)
 {
     void *sPtr, *sBase, *rPtr, *rBase;
 
     if(sendBuf == NULL)
         sPtr = MPI_IN_PLACE;
     else
-        sPtr = ompi_java_getBufPtr(&sBase, env, sendBuf, baseType, sendOff);
+        sPtr = ompi_java_getBufPtr(&sBase, env, sendBuf, sdb, baseType, sendOff);
 
-    rPtr = ompi_java_getBufPtr(&rBase, env, recvBuf, baseType, recvOff);
+    rPtr = ompi_java_getBufPtr(&rBase, env, recvBuf, rdb, baseType, recvOff);
 
     int rc = MPI_Scan(sPtr, rPtr, count, (MPI_Datatype)type,
-                      ompi_java_op_getHandle(env, op, baseType),
+                      ompi_java_op_getHandle(env, jOp, hOp, baseType),
                       (MPI_Comm)comm);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, recvBuf, rBase, baseType);
+    ompi_java_releaseBufPtr(env, recvBuf, rdb, rBase, baseType);
 
     if(sendBuf != NULL)
-        ompi_java_releaseReadBufPtr(env, sendBuf, sBase, baseType);
+        ompi_java_releaseReadBufPtr(env, sendBuf, sdb, sBase, baseType);
 }
 
 JNIEXPORT jlong JNICALL Java_mpi_Intracomm_iScan(
         JNIEnv *env, jobject jthis, jlong comm,
         jobject sendBuf, jobject recvBuf, jint count,
-        jlong type, int baseType, jobject op)
+        jlong type, int baseType, jobject jOp, jlong hOp)
 {
     void *sPtr, *rPtr;
     MPI_Request request;
@@ -216,7 +217,7 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_iScan(
     rPtr = (*env)->GetDirectBufferAddress(env, recvBuf);
 
     int rc = MPI_Iscan(sPtr, rPtr, count, (MPI_Datatype)type,
-                       ompi_java_op_getHandle(env, op, baseType),
+                       ompi_java_op_getHandle(env, jOp, hOp, baseType),
                        (MPI_Comm)comm, &request);
 
     ompi_java_exceptionCheck(env, rc);
@@ -225,33 +226,34 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_iScan(
 
 JNIEXPORT void JNICALL Java_mpi_Intracomm_exScan(
         JNIEnv *env, jobject jthis, jlong comm,
-        jobject sendBuf, jint sendOff, jobject recvBuf, jint recvOff,
-        jint count, jlong type, int bType, jobject op)
+        jobject sendBuf, jboolean sdb, jint sendOff,
+        jobject recvBuf, jboolean rdb, jint recvOff, jint count,
+        jlong type, int bType, jobject jOp, jlong hOp)
 {
     void *sPtr, *sBase, *rPtr, *rBase;
 
     if(sendBuf == NULL)
         sPtr = MPI_IN_PLACE;
     else
-        sPtr = ompi_java_getBufPtr(&sBase, env, sendBuf, bType, sendOff);
+        sPtr = ompi_java_getBufPtr(&sBase, env, sendBuf, sdb, bType, sendOff);
 
-    rPtr = ompi_java_getBufPtr(&rBase, env, recvBuf, bType, recvOff);
+    rPtr = ompi_java_getBufPtr(&rBase, env, recvBuf, rdb, bType, recvOff);
 
     int rc = MPI_Exscan(sPtr, rPtr, count, (MPI_Datatype)type,
-                        ompi_java_op_getHandle(env, op, bType),
+                        ompi_java_op_getHandle(env, jOp, hOp, bType),
                         (MPI_Comm)comm);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, recvBuf, rBase, bType);
+    ompi_java_releaseBufPtr(env, recvBuf, rdb, rBase, bType);
 
     if(sendBuf != NULL)
-        ompi_java_releaseReadBufPtr(env, sendBuf, sBase, bType);
+        ompi_java_releaseReadBufPtr(env, sendBuf, sdb, sBase, bType);
 }
 
 JNIEXPORT jlong JNICALL Java_mpi_Intracomm_iExScan(
         JNIEnv *env, jobject jthis, jlong comm,
         jobject sendBuf, jobject recvBuf, jint count,
-        jlong type, int bType, jobject op)
+        jlong type, int bType, jobject jOp, jlong hOp)
 {
     void *sPtr, *rPtr;
 
@@ -264,7 +266,7 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_iExScan(
     MPI_Request request;
 
     int rc = MPI_Iexscan(sPtr, rPtr, count, (MPI_Datatype)type,
-                         ompi_java_op_getHandle(env, op, bType),
+                         ompi_java_op_getHandle(env, jOp, hOp, bType),
                          (MPI_Comm)comm, &request);
 
     ompi_java_exceptionCheck(env, rc);
