@@ -18,12 +18,6 @@ import static mpi.MPI.assertDirectBuffer;
  */
 public final class File
 {
-// Auxiliary status data.
-// It's used to avoid creating status objects in the C side.
-// It also avoids setting objects attributes in the C side.
-// Calling java methods and setting object attributes from C is very slow.
-private long[] status = Status.newData();
-
 private long handle;
 private FileView view = new FileView(0, MPI.BYTE, MPI.BYTE, "native");
 private Status beginStatus;
@@ -251,6 +245,7 @@ public Status readAt(long offset, Object buf, int count, Datatype type)
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -259,9 +254,9 @@ public Status readAt(long offset, Object buf, int count, Datatype type)
     }
 
     readAt(handle, offset, buf, db, off, count,
-           type.handle, type.baseType, status);
+           type.handle, type.baseType, status.data);
 
-    return newStatus();
+    return status;
 }
 
 private native void readAt(
@@ -283,6 +278,7 @@ public Status readAtAll(long offset, Object buf, int count, Datatype type)
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -291,9 +287,9 @@ public Status readAtAll(long offset, Object buf, int count, Datatype type)
     }
 
     readAtAll(handle, offset, buf, db, off, count,
-              type.handle, type.baseType, status);
+              type.handle, type.baseType, status.data);
 
-    return newStatus();
+    return status;
 }
 
 private native void readAtAll(
@@ -315,6 +311,7 @@ public Status writeAt(long offset, Object buf, int count, Datatype type)
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -323,9 +320,9 @@ public Status writeAt(long offset, Object buf, int count, Datatype type)
     }
 
     writeAt(handle, offset, buf, db, off, count,
-            type.handle, type.baseType, status);
+            type.handle, type.baseType, status.data);
 
-    return newStatus();
+    return status;
 }
 
 private native void writeAt(
@@ -347,6 +344,7 @@ public Status writeAtAll(long offset, Object buf, int count, Datatype type)
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -355,9 +353,9 @@ public Status writeAtAll(long offset, Object buf, int count, Datatype type)
     }
 
     writeAtAll(handle, offset, buf, db, off, count,
-               type.handle, type.baseType, status);
+               type.handle, type.baseType, status.data);
 
-    return newStatus();
+    return status;
 }
 
 private native void writeAtAll(
@@ -419,6 +417,7 @@ public Status read(Object buf, int count, Datatype type) throws MPIException
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -426,8 +425,8 @@ public Status read(Object buf, int count, Datatype type) throws MPIException
         buf = ((Buffer)buf).array();
     }
 
-    read(handle, buf, db, off, count, type.handle, type.baseType, status);
-    return newStatus();
+    read(handle, buf, db, off, count, type.handle, type.baseType, status.data);
+    return status;
 }
 
 private native void read(
@@ -447,6 +446,7 @@ public Status readAll(Object buf, int count, Datatype type) throws MPIException
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -454,8 +454,8 @@ public Status readAll(Object buf, int count, Datatype type) throws MPIException
         buf = ((Buffer)buf).array();
     }
 
-    readAll(handle, buf, db, off, count, type.handle, type.baseType, status);
-    return newStatus();
+    readAll(handle, buf,db,off, count, type.handle, type.baseType, status.data);
+    return status;
 }
 
 private native void readAll(
@@ -475,6 +475,7 @@ public Status write(Object buf, int count, Datatype type) throws MPIException
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -482,8 +483,8 @@ public Status write(Object buf, int count, Datatype type) throws MPIException
         buf = ((Buffer)buf).array();
     }
 
-    write(handle, buf, db, off, count, type.handle, type.baseType, status);
-    return newStatus();
+    write(handle, buf, db, off, count, type.handle, type.baseType, status.data);
+    return status;
 }
 
 private native void write(
@@ -503,6 +504,7 @@ public Status writeAll(Object buf, int count, Datatype type) throws MPIException
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -510,8 +512,8 @@ public Status writeAll(Object buf, int count, Datatype type) throws MPIException
         buf = ((Buffer)buf).array();
     }
 
-    writeAll(handle, buf,db,off, count, type.handle, type.baseType, status);
-    return newStatus();
+    writeAll(handle, buf,db,off, count, type.handle,type.baseType, status.data);
+    return status;
 }
 
 private native void writeAll(
@@ -609,6 +611,7 @@ public Status readShared(Object buf, int count, Datatype type)
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -616,8 +619,10 @@ public Status readShared(Object buf, int count, Datatype type)
         buf = ((Buffer)buf).array();
     }
 
-    readShared(handle, buf, db, off, count, type.handle, type.baseType, status);
-    return newStatus();
+    readShared(handle, buf, db, off, count,
+               type.handle, type.baseType, status.data);
+
+    return status;
 }
 
 private native void readShared(
@@ -638,6 +643,7 @@ public Status writeShared(Object buf, int count, Datatype type)
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -645,8 +651,10 @@ public Status writeShared(Object buf, int count, Datatype type)
         buf = ((Buffer)buf).array();
     }
 
-    writeShared(handle, buf,db,off, count, type.handle, type.baseType, status);
-    return newStatus();
+    writeShared(handle, buf, db, off, count,
+                type.handle, type.baseType, status.data);
+
+    return status;
 }
 
 private native void writeShared(
@@ -705,6 +713,7 @@ public Status readOrdered(Object buf, int count, Datatype type)
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -712,8 +721,10 @@ public Status readOrdered(Object buf, int count, Datatype type)
         buf = ((Buffer)buf).array();
     }
 
-    readOrdered(handle, buf,db,off, count, type.handle, type.baseType, status);
-    return newStatus();
+    readOrdered(handle, buf, db, off, count,
+                type.handle, type.baseType, status.data);
+
+    return status;
 }
 
 private native void readOrdered(
@@ -734,6 +745,7 @@ public Status writeOrdered(Object buf, int count, Datatype type)
     MPI.check();
     int off = 0;
     boolean db = false;
+    Status status = new Status();
 
     if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
@@ -741,8 +753,10 @@ public Status writeOrdered(Object buf, int count, Datatype type)
         buf = ((Buffer)buf).array();
     }
 
-    writeOrdered(handle, buf,db,off, count, type.handle, type.baseType, status);
-    return newStatus();
+    writeOrdered(handle, buf, db, off, count,
+                 type.handle, type.baseType, status.data);
+
+    return status;
 }
 
 private native void writeOrdered(
@@ -797,6 +811,7 @@ public void readAtAllBegin(long offset, Object buf, int count, Datatype type)
     else
     {
         int off = 0;
+        Status status = new Status();
 
         if(isHeapBuffer(buf))
         {
@@ -805,9 +820,9 @@ public void readAtAllBegin(long offset, Object buf, int count, Datatype type)
         }
 
         readAtAll(handle, offset, buf, false, off, count,
-                  type.handle, type.baseType, status);
+                  type.handle, type.baseType, status.data);
 
-        beginStatus = newStatus();
+        beginStatus = status;
     }
 }
 
@@ -827,8 +842,9 @@ public Status readAtAllEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        readAtAllEnd(handle, buf, status);
-        return newStatus();
+        Status status = new Status();
+        readAtAllEnd(handle, buf, status.data);
+        return status;
     }
     else
     {
@@ -859,6 +875,7 @@ public void writeAtAllBegin(long offset, Object buf, int count, Datatype type)
     else
     {
         int off = 0;
+        Status status = new Status();
 
         if(isHeapBuffer(buf))
         {
@@ -867,9 +884,9 @@ public void writeAtAllBegin(long offset, Object buf, int count, Datatype type)
         }
 
         writeAtAll(handle, offset, buf, false, off, count,
-                   type.handle, type.baseType, status);
+                   type.handle, type.baseType, status.data);
 
-        beginStatus = newStatus();
+        beginStatus = status;
     }
 }
 
@@ -889,8 +906,9 @@ public Status writeAtAllEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        writeAtAllEnd(handle, buf, status);
-        return newStatus();
+        Status status = new Status();
+        writeAtAllEnd(handle, buf, status.data);
+        return status;
     }
     else
     {
@@ -920,6 +938,7 @@ public void readAllBegin(Object buf, int count, Datatype type)
     else
     {
         int off = 0;
+        Status status = new Status();
 
         if(isHeapBuffer(buf))
         {
@@ -928,9 +947,9 @@ public void readAllBegin(Object buf, int count, Datatype type)
         }
 
         readAll(handle, buf, false, off, count,
-                type.handle, type.baseType, status);
+                type.handle, type.baseType, status.data);
 
-        beginStatus = newStatus();
+        beginStatus = status;
     }
 }
 
@@ -949,8 +968,9 @@ public Status readAllEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        readAllEnd(handle, buf, status);
-        return newStatus();
+        Status status = new Status();
+        readAllEnd(handle, buf, status.data);
+        return status;
     }
     else
     {
@@ -980,6 +1000,7 @@ public void writeAllBegin(Object buf, int count, Datatype type)
     else
     {
         int off = 0;
+        Status status = new Status();
 
         if(isHeapBuffer(buf))
         {
@@ -988,9 +1009,9 @@ public void writeAllBegin(Object buf, int count, Datatype type)
         }
 
         writeAll(handle, buf, false, off, count,
-                 type.handle, type.baseType, status);
+                 type.handle, type.baseType, status.data);
 
-        beginStatus = newStatus();
+        beginStatus = status;
     }
 }
 
@@ -1009,8 +1030,9 @@ public Status writeAllEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        writeAllEnd(handle, buf, status);
-        return newStatus();
+        Status status = new Status();
+        writeAllEnd(handle, buf, status.data);
+        return status;
     }
     else
     {
@@ -1040,6 +1062,7 @@ public void readOrderedBegin(Object buf, int count, Datatype type)
     else
     {
         int off = 0;
+        Status status = new Status();
 
         if(isHeapBuffer(buf))
         {
@@ -1048,9 +1071,9 @@ public void readOrderedBegin(Object buf, int count, Datatype type)
         }
 
         readOrdered(handle, buf, false, off, count,
-                    type.handle, type.baseType, status);
+                    type.handle, type.baseType, status.data);
 
-        beginStatus = newStatus();
+        beginStatus = status;
     }
 }
 
@@ -1069,8 +1092,9 @@ public Status readOrderedEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        readOrderedEnd(handle, buf, status);
-        return newStatus();
+        Status status = new Status();
+        readOrderedEnd(handle, buf, status.data);
+        return status;
     }
     else
     {
@@ -1100,6 +1124,7 @@ public void writeOrderedBegin(Object buf, int count, Datatype type)
     else
     {
         int off = 0;
+        Status status = new Status();
 
         if(isHeapBuffer(buf))
         {
@@ -1108,9 +1133,9 @@ public void writeOrderedBegin(Object buf, int count, Datatype type)
         }
 
         writeOrdered(handle, buf, false, off, count,
-                     type.handle, type.baseType, status);
+                     type.handle, type.baseType, status.data);
 
-        beginStatus = newStatus();
+        beginStatus = status;
     }
 }
 
@@ -1129,8 +1154,9 @@ public Status writeOrderedEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        writeOrderedEnd(handle, buf, status);
-        return newStatus();
+        Status status = new Status();
+        writeOrderedEnd(handle, buf, status.data);
+        return status;
     }
     else
     {
@@ -1188,11 +1214,4 @@ public void sync() throws MPIException
 
 private native void sync(long handle) throws MPIException;
 
-private Status newStatus()
-{
-    Status s = new Status(status);
-    status = Status.newData();
-    return s;
-}
-
 } // File
