@@ -30,6 +30,14 @@ public final class Status
 {
 protected final long[] data;
 
+/**
+ * Status objects must be created only by the MPI methods.
+ */
+protected Status()
+{
+    data = new long[6];
+}
+
 /**
  * Status objects must be created only by the MPI methods.
  */
