@@ -10,7 +10,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2013 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2012 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * Copyright (c) 2006      University of Houston. All rights reserved.
@@ -60,7 +60,9 @@
 #include "ompi/runtime/mpiruntime.h"
 #include "ompi/attribute/attribute.h"
 #include "ompi/mca/pml/pml.h"
+#include "ompi/mca/bml/bml.h"
 #include "ompi/mca/pml/base/base.h"
+#include "ompi/mca/bml/base/base.h"
 #include "ompi/mca/osc/base/base.h"
 #include "ompi/mca/coll/base/base.h"
 #include "ompi/mca/rte/rte.h"
@@ -92,6 +94,8 @@ int ompi_mpi_finalize(void)
     opal_list_item_t *item;
     struct timeval ompistart, ompistop;
     ompi_rte_collective_t *coll;
+    ompi_proc_t** procs;
+    size_t nprocs;
 
     /* Be a bit social if an erroneous program calls MPI_FINALIZE in
        two different threads, otherwise we may deadlock in
@@ -114,11 +118,6 @@ int ompi_mpi_finalize(void)
 
     ompi_mpiext_fini();
 
-    /* As finalize is the last legal MPI call, we are allowed to force the release
-     * of the user buffer used for bsend, before going anywhere further.
-     */
-    (void)mca_pml_base_bsend_detach(NULL, NULL);
-
     /* Per MPI-2:4.8, we have to free MPI_COMM_SELF before doing
        anything else in MPI_FINALIZE (to include setting up such that
        MPI_FINALIZED will return true). */
@@ -134,6 +133,16 @@ int ompi_mpi_finalize(void)
 
     ompi_mpi_finalized = true;
 
+    /* As finalize is the last legal MPI call, we are allowed to force the release
+     * of the user buffer used for bsend, before going anywhere further.
+     */
+    (void)mca_pml_base_bsend_detach(NULL, NULL);
+
+    nprocs = 0;
+    procs = ompi_proc_all(&nprocs);
+    MCA_PML_CALL(del_procs(procs, nprocs));
+    free(procs);
+
 #if OMPI_ENABLE_PROGRESS_THREADS == 0
     opal_progress_set_event_flag(OPAL_EVLOOP_ONCE | OPAL_EVLOOP_NONBLOCK);
 #endif
@@ -393,6 +402,9 @@ int ompi_mpi_finalize(void)
     if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_coll_base_framework))) {
         return ret;
     }
+    if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_bml_base_framework))) {
+        return ret;
+    }
     if (OMPI_SUCCESS != (ret = mca_base_framework_close(&ompi_mpool_base_framework))) {
         return ret;
     }
@@ -400,6 +412,11 @@ int ompi_mpi_finalize(void)
         return ret;
     }
 
+    if (NULL != ompi_mpi_main_thread) {
+        OBJ_RELEASE(ompi_mpi_main_thread);
+        ompi_mpi_main_thread = NULL;
+    }
+
     /* Leave the RTE */
 
     if (OMPI_SUCCESS != (ret = ompi_rte_finalize())) {
