@@ -107,7 +107,27 @@ static int init(void)
 
 static void finalize(void)
 {
-    opal_hash_table_remove_all(&hash_data);
+    proc_data_t *proc_data;
+    uint64_t key;
+    char *node;
+
+    /* to assist in getting a clean valgrind, cycle thru the hash table
+     * and release all data stored in it
+     */
+    if (OPAL_SUCCESS == opal_hash_table_get_first_key_uint64(&hash_data, &key,
+                                                             (void**)&proc_data,
+                                                             (void**)&node)) {
+        if (NULL != proc_data) {
+            OBJ_RELEASE(proc_data);
+        }
+        while (OPAL_SUCCESS == opal_hash_table_get_next_key_uint64(&hash_data, &key,
+                                                                   (void**)&proc_data,
+                                                                   node, (void**)&node)) {
+            if (NULL != proc_data) {
+                OBJ_RELEASE(proc_data);
+            }
+        }
+    }
     OBJ_DESTRUCT(&hash_data);
 }
 
@@ -220,6 +240,14 @@ static int store(const opal_identifier_t *uid,
             kv->data.string = NULL;
         }
         break;
+    case OPAL_UINT64:
+        if (NULL == data) {
+            OPAL_ERROR_LOG(OPAL_ERR_BAD_PARAM);
+            return OPAL_ERR_BAD_PARAM;
+        }
+        kv->type = OPAL_UINT64;
+        kv->data.uint64 = *(uint64_t*)(data);
+        break;
     case OPAL_UINT32:
         if (NULL == data) {
             OPAL_ERROR_LOG(OPAL_ERR_BAD_PARAM);
@@ -252,6 +280,14 @@ static int store(const opal_identifier_t *uid,
         kv->type = OPAL_UINT;
         kv->data.uint = *(unsigned int*)(data);
         break;
+    case OPAL_FLOAT:
+        if (NULL == data) {
+            OPAL_ERROR_LOG(OPAL_ERR_BAD_PARAM);
+            return OPAL_ERR_BAD_PARAM;
+        }
+        kv->type = OPAL_FLOAT;
+        kv->data.fval = *(float*)(data);
+        break;
     case OPAL_BYTE_OBJECT:
         kv->type = OPAL_BYTE_OBJECT;
         boptr = (opal_byte_object_t*)data;
@@ -315,6 +351,7 @@ static int store_pointer(const opal_identifier_t *uid,
         opal_list_remove_item(&proc_data->data, &k2->super);
         OBJ_RELEASE(k2);
     }
+    kv->scope |= OPAL_SCOPE_REFER;  // mark that this value was stored by reference and doesn't belong to us
     opal_list_append(&proc_data->data, &kv->super);
     return OPAL_SUCCESS;
 }
@@ -371,6 +408,12 @@ static int fetch(const opal_identifier_t *uid,
             *data = NULL;
         }
         break;
+    case OPAL_UINT64:
+        if (OPAL_UINT64 != kv->type) {
+            return OPAL_ERR_TYPE_MISMATCH;
+        }
+        memcpy(*data, &kv->data.uint64, 8);
+        break;
     case OPAL_UINT32:
         if (OPAL_UINT32 != kv->type) {
             return OPAL_ERR_TYPE_MISMATCH;
@@ -395,6 +438,12 @@ static int fetch(const opal_identifier_t *uid,
         }
         memcpy(*data, &kv->data.uint, sizeof(unsigned int));
         break;
+    case OPAL_FLOAT:
+        if (OPAL_FLOAT != kv->type) {
+            return OPAL_ERR_TYPE_MISMATCH;
+        }
+        memcpy(*data, &kv->data.fval, sizeof(float));
+        break;
     case OPAL_BYTE_OBJECT:
         if (OPAL_BYTE_OBJECT != kv->type) {
             return OPAL_ERR_TYPE_MISMATCH;
@@ -458,6 +507,12 @@ static int fetch_pointer(const opal_identifier_t *uid,
         }
         *data = kv->data.string;
         break;
+    case OPAL_UINT64:
+        if (OPAL_UINT64 != kv->type) {
+            return OPAL_ERR_TYPE_MISMATCH;
+        }
+        *data = &kv->data.uint64;
+        break;
     case OPAL_UINT32:
         if (OPAL_UINT32 != kv->type) {
             return OPAL_ERR_TYPE_MISMATCH;
@@ -488,6 +543,12 @@ static int fetch_pointer(const opal_identifier_t *uid,
         }
         *data = &kv->data.bo;
         break;
+    case OPAL_FLOAT:
+        if (OPAL_FLOAT != kv->type) {
+            return OPAL_ERR_TYPE_MISMATCH;
+        }
+        *data = &kv->data.fval;
+        break;
     default:
         OPAL_ERROR_LOG(OPAL_ERR_NOT_SUPPORTED);
         return OPAL_ERR_NOT_SUPPORTED;
@@ -599,7 +660,10 @@ static int remove_data(const opal_identifier_t *uid, const char *key)
          kv != (opal_value_t*) opal_list_get_end(&proc_data->data);
          kv = (opal_value_t*) opal_list_get_next(kv)) {
         if (0 == strcmp(key, kv->key)) {
-            OBJ_RELEASE(kv);
+            opal_list_remove_item(&proc_data->data, &kv->super);
+            if (!(kv->scope & OPAL_SCOPE_REFER)) {
+                OBJ_RELEASE(kv);
+            }
             break;
         }
     }
