@@ -12,7 +12,7 @@
  * Copyright (c) 2011-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013      Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2014 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -48,7 +48,7 @@
  * only find PUs (!).  On such platforms, then do the same calculation
  * but with PUs instead of COREs.
  */
-static hwloc_obj_t get_pu(hwloc_topology_t topo,  int lid)
+hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,  int lid)
 {
     hwloc_obj_type_t obj_type = HWLOC_OBJ_CORE;
     hwloc_obj_t obj;
@@ -130,7 +130,7 @@ int opal_hwloc_base_filter_cpus(hwloc_topology_t topo)
             case 1:
                 /* only one cpu given - get that object */
                 cpu = strtoul(range[0], NULL, 10);
-                if (NULL == (pu = get_pu(topo, cpu))) {
+                if (NULL == (pu = opal_hwloc_base_get_pu(topo, cpu))) {
                     opal_argv_free(ranges);
                     opal_argv_free(range);
                     return OPAL_ERROR;
@@ -144,7 +144,7 @@ int opal_hwloc_base_filter_cpus(hwloc_topology_t topo)
                 start = strtoul(range[0], NULL, 10);
                 end = strtoul(range[1], NULL, 10);
                 for (cpu=start; cpu <= end; cpu++) {
-                    if (NULL == (pu = get_pu(topo, cpu))) {
+                    if (NULL == (pu = opal_hwloc_base_get_pu(topo, cpu))) {
                         opal_argv_free(ranges);
                         opal_argv_free(range);
                         hwloc_bitmap_free(avail);
@@ -1265,7 +1265,7 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
                 for (j=0; NULL != list[j]; j++) {
                     core_id = atoi(list[j]);
                     /* find the specified logical available cpu */
-                    if (NULL == (pu = get_pu(topo, core_id))) {
+                    if (NULL == (pu = opal_hwloc_base_get_pu(topo, core_id))) {
                         opal_argv_free(range);
                         opal_argv_free(item);
                         return OPAL_ERROR;
@@ -1283,7 +1283,7 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
                 upper_range = atoi(range[1]);
                 for (core_id=lower_range; core_id <= upper_range; core_id++) {
                     /* find the specified logical available cpu */
-                    if (NULL == (pu = get_pu(topo, core_id))) {
+                    if (NULL == (pu = opal_hwloc_base_get_pu(topo, core_id))) {
                         opal_argv_free(range);
                         opal_argv_free(item);
                         return OPAL_ERROR;
@@ -1413,6 +1413,8 @@ opal_hwloc_locality_t opal_hwloc_base_get_relative_locality(hwloc_topology_t top
     opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
                         "locality: %s",
                         opal_hwloc_base_print_locality(locality));
+    hwloc_bitmap_free(loc1);
+    hwloc_bitmap_free(loc2);
 
     return locality;
 }
@@ -1671,7 +1673,7 @@ static char *bitmap2rangestr(int bitmap)
  * Make a map of socket/core/hwthread tuples
  */
 static int build_map(int *num_sockets_arg, int *num_cores_arg, 
-                     hwloc_cpuset_t cpuset, int ***map)
+                     hwloc_cpuset_t cpuset, int ***map, hwloc_topology_t topo)
 {
     static int num_sockets = -1, num_cores = -1;
     int socket_index, core_index, pu_index;
@@ -1681,7 +1683,7 @@ static int build_map(int *num_sockets_arg, int *num_cores_arg,
     /* Find out how many sockets we have (cached so that we don't have
        to look this up every time) */
     if (num_sockets < 0) {
-        num_sockets = hwloc_get_nbobjs_by_type(opal_hwloc_topology, HWLOC_OBJ_SOCKET);
+        num_sockets = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_SOCKET);
         /* some systems (like the iMac) only have one
          * socket and so don't report a socket
          */
@@ -1691,7 +1693,7 @@ static int build_map(int *num_sockets_arg, int *num_cores_arg,
         /* Lazy: take the total number of cores that we have in the
            topology; that'll be more than the max number of cores
            under any given socket */
-        num_cores = hwloc_get_nbobjs_by_type(opal_hwloc_topology, HWLOC_OBJ_CORE);
+        num_cores = hwloc_get_nbobjs_by_type(topo, HWLOC_OBJ_CORE);
     }
     *num_sockets_arg = num_sockets;
     *num_cores_arg = num_cores;
@@ -1713,11 +1715,11 @@ static int build_map(int *num_sockets_arg, int *num_cores_arg,
     /* Iterate the PUs in this cpuset; fill in the data[][] array with
        the socket/core/pu triples */
     for (pu_index = 0,
-             pu = hwloc_get_obj_inside_cpuset_by_type(opal_hwloc_topology,
+             pu = hwloc_get_obj_inside_cpuset_by_type(topo,
                                                       cpuset, HWLOC_OBJ_PU, 
                                                       pu_index);
          NULL != pu;
-         pu = hwloc_get_obj_inside_cpuset_by_type(opal_hwloc_topology,
+         pu = hwloc_get_obj_inside_cpuset_by_type(topo,
                                                   cpuset, HWLOC_OBJ_PU, 
                                                   ++pu_index)) {
         /* Go upward and find the core this PU belongs to */
@@ -1752,7 +1754,9 @@ static int build_map(int *num_sockets_arg, int *num_cores_arg,
 /*
  * Make a prettyprint string for a hwloc_cpuset_t
  */
-int opal_hwloc_base_cset2str(char *str, int len, hwloc_cpuset_t cpuset)
+int opal_hwloc_base_cset2str(char *str, int len,
+                             hwloc_topology_t topo,
+                             hwloc_cpuset_t cpuset)
 {
     bool first;
     int num_sockets, num_cores;
@@ -1771,9 +1775,9 @@ int opal_hwloc_base_cset2str(char *str, int len, hwloc_cpuset_t cpuset)
     }
 
     /* if the cpuset includes all available cpus, then we are unbound */
-    root = hwloc_get_root_obj(opal_hwloc_topology);
+    root = hwloc_get_root_obj(topo);
     if (NULL == root->userdata) {
-        opal_hwloc_base_filter_cpus(opal_hwloc_topology);
+        opal_hwloc_base_filter_cpus(topo);
     }
     sum = (opal_hwloc_topo_data_t*)root->userdata;
     if (NULL == sum->available) {
@@ -1783,7 +1787,7 @@ int opal_hwloc_base_cset2str(char *str, int len, hwloc_cpuset_t cpuset)
         return OPAL_ERR_NOT_BOUND;
     }
 
-    if (OPAL_SUCCESS != (ret = build_map(&num_sockets, &num_cores, cpuset, &map))) {
+    if (OPAL_SUCCESS != (ret = build_map(&num_sockets, &num_cores, cpuset, &map, topo))) {
         return ret;
     }
 
@@ -1818,7 +1822,9 @@ int opal_hwloc_base_cset2str(char *str, int len, hwloc_cpuset_t cpuset)
  *        . - signifies PU a process not bound to
  *        B - signifies PU a process is bound to
  */
-int opal_hwloc_base_cset2mapstr(char *str, int len, hwloc_cpuset_t cpuset)
+int opal_hwloc_base_cset2mapstr(char *str, int len,
+                                hwloc_topology_t topo,
+                                hwloc_cpuset_t cpuset)
 {
     char tmp[BUFSIZ];
     int core_index, pu_index;
@@ -1835,9 +1841,9 @@ int opal_hwloc_base_cset2mapstr(char *str, int len, hwloc_cpuset_t cpuset)
     }
 
     /* if the cpuset includes all available cpus, then we are unbound */
-    root = hwloc_get_root_obj(opal_hwloc_topology);
+    root = hwloc_get_root_obj(topo);
     if (NULL == root->userdata) {
-        opal_hwloc_base_filter_cpus(opal_hwloc_topology);
+        opal_hwloc_base_filter_cpus(topo);
     }
     sum = (opal_hwloc_topo_data_t*)root->userdata;
     if (NULL == sum->available) {
@@ -1848,19 +1854,18 @@ int opal_hwloc_base_cset2mapstr(char *str, int len, hwloc_cpuset_t cpuset)
     }
 
     /* Iterate over all existing sockets */
-    for (socket = hwloc_get_obj_by_type(opal_hwloc_topology, 
-                                        HWLOC_OBJ_SOCKET, 0);
+    for (socket = hwloc_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0);
          NULL != socket; 
          socket = socket->next_cousin) {
         strncat(str, "[", len - strlen(str));
 
         /* Iterate over all existing cores in this socket */
         core_index = 0;
-        for (core = hwloc_get_obj_inside_cpuset_by_type(opal_hwloc_topology,
+        for (core = hwloc_get_obj_inside_cpuset_by_type(topo,
                                                         socket->cpuset, 
                                                         HWLOC_OBJ_CORE, core_index);
              NULL != core; 
-             core = hwloc_get_obj_inside_cpuset_by_type(opal_hwloc_topology,
+             core = hwloc_get_obj_inside_cpuset_by_type(topo,
                                                         socket->cpuset, 
                                                         HWLOC_OBJ_CORE, ++core_index)) {
             if (core_index > 0) {
@@ -1869,11 +1874,11 @@ int opal_hwloc_base_cset2mapstr(char *str, int len, hwloc_cpuset_t cpuset)
 
             /* Iterate over all existing PUs in this core */
             pu_index = 0;
-            for (pu = hwloc_get_obj_inside_cpuset_by_type(opal_hwloc_topology,
+            for (pu = hwloc_get_obj_inside_cpuset_by_type(topo,
                                                           core->cpuset, 
                                                           HWLOC_OBJ_PU, pu_index);
                  NULL != pu; 
-                 pu = hwloc_get_obj_inside_cpuset_by_type(opal_hwloc_topology,
+                 pu = hwloc_get_obj_inside_cpuset_by_type(topo,
                                                           core->cpuset, 
                                                           HWLOC_OBJ_PU, ++pu_index)) {
 
