@@ -15,7 +15,7 @@
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2010-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013      Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2014 Intel, Inc. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -28,6 +28,7 @@
 #include "opal_config.h"
 
 #include "opal/util/malloc.h"
+#include "opal/util/arch.h"
 #include "opal/util/output.h"
 #include "opal/util/show_help.h"
 #include "opal/memoryhooks/memory.h"
@@ -39,6 +40,7 @@
 #include "opal/mca/memory/base/base.h"
 #include "opal/mca/memcpy/base/base.h"
 #include "opal/mca/hwloc/base/base.h"
+#include "opal/mca/sec/base/base.h"
 #include "opal/mca/timer/base/base.h"
 #include "opal/mca/memchecker/base/base.h"
 #include "opal/dss/dss.h"
@@ -224,6 +226,12 @@ opal_err2str(int errnum, const char **errmsg)
     case OPAL_ERR_DATA_VALUE_NOT_FOUND:
         retval = "Data for specified key not found";
         break;
+    case OPAL_ERR_CONNECTION_FAILED:
+        retval = "Connection failed";
+        break;
+    case OPAL_ERR_AUTHENTICATION_FAILED:
+        retval = "Authentication failed";
+        break;
     default:
         retval = NULL;
     }
@@ -478,6 +486,16 @@ opal_init(int* pargc, char*** pargv)
         goto return_error;
     }
     
+    /* initialize the security framework */
+    if( OPAL_SUCCESS != (ret = mca_base_framework_open(&opal_sec_base_framework, 0)) ) {
+        error = "opal_sec_base_open";
+        goto return_error;
+    }
+    if( OPAL_SUCCESS != (ret = opal_sec_base_select()) ) {
+        error = "opal_sec_base_select";
+        goto return_error;
+    }
+
     return OPAL_SUCCESS;
 
  return_error:
