@@ -12,6 +12,7 @@
  * Copyright (c) 2011-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
+ * Copyright (c) 2014      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -29,6 +30,7 @@
 #include <errno.h>
 
 #include "opal/util/output.h"
+#include "opal/mca/db/db.h"
 #include "opal/mca/hwloc/base/base.h"
 
 #include "orte/mca/errmgr/errmgr.h"
@@ -264,7 +266,7 @@ int orte_ess_base_proc_binding(void)
             free(orte_process_info.cpuset);
             orte_process_info.cpuset = NULL;
         }
-        if (opal_hwloc_report_bindings) {
+        if (opal_hwloc_report_bindings || 4 < opal_output_get_verbosity(orte_ess_base_framework.framework_output)) {
             opal_output(0, "MCW rank %d is not bound",
                         ORTE_PROC_MY_NAME->vpid);
         }
@@ -276,18 +278,27 @@ int orte_ess_base_proc_binding(void)
         }
         hwloc_bitmap_list_asprintf(&orte_process_info.cpuset, mycpus);
         /* report the binding, if requested */
-        if (opal_hwloc_report_bindings) {
+        if (opal_hwloc_report_bindings || 4 < opal_output_get_verbosity(orte_ess_base_framework.framework_output)) {
             char tmp1[1024], tmp2[1024];
-            if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), mycpus)) {
+            if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), opal_hwloc_topology, mycpus)) {
                 opal_output(0, "MCW rank %d is not bound (or bound to all available processors)", ORTE_PROC_MY_NAME->vpid);
             } else {
-                opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), mycpus);
+                opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), opal_hwloc_topology, mycpus);
                 opal_output(0, "MCW rank %d bound to %s: %s",
                             ORTE_PROC_MY_NAME->vpid, tmp1, tmp2);
             }
         }
     }
     hwloc_bitmap_free(mycpus);
+    /* store our cpuset for exchange with non-peers
+     * so that other procs in a comm_spawn can know it
+     */
+    if (ORTE_SUCCESS != (ret = opal_db.store((opal_identifier_t*)ORTE_PROC_MY_NAME,
+                                             OPAL_SCOPE_NON_PEER,
+                                             OPAL_DB_CPUSET, orte_process_info.cpuset, OPAL_STRING))) {
+        ORTE_ERROR_LOG(ret);
+        goto error;
+    }
 
     return ORTE_SUCCESS;
 
