@@ -70,6 +70,7 @@ static int orte_oob_base_close(void)
 
     /* destruct our internal lists */
     OBJ_DESTRUCT(&orte_oob_base.actives);
+    OBJ_DESTRUCT(&orte_oob_base.peers);
 
     return mca_base_framework_components_close(&orte_oob_base_framework, NULL);
 }
@@ -95,8 +96,8 @@ MCA_BASE_FRAMEWORK_DECLARE(orte, oob, "Out-of-Band Messaging Subsystem",
                            mca_oob_base_static_components, 0);
 
 
-OBJ_CLASS_INSTANCE(mca_oob_base_component_t,
-                   opal_list_item_t,
+OBJ_CLASS_INSTANCE(orte_oob_send_t,
+                   opal_object_t,
                    NULL, NULL);
 
 static void pr_cons(orte_oob_base_peer_t *ptr)
