@@ -379,15 +379,6 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
     }
 
     if (1 < orte_rmaps_base.cpus_per_rank) {
-        /* check to see if we were told to map at too low a level */
-        if ((ORTE_MAPPING_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping)) &&
-            ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) > ORTE_MAPPING_BYSOCKET &&
-            ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) < ORTE_MAPPING_BYSLOT) {
-            orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low", true,
-                           orte_rmaps_base.cpus_per_rank,
-                           orte_rmaps_base_print_mapping(orte_rmaps_base.mapping));
-            return ORTE_ERR_SILENT;
-        }
 #if OPAL_HAVE_HWLOC
         /* if we were asked for multiple cpus/proc, then we have to
          * bind to those cpus - any other binding policy is an
