@@ -601,6 +601,12 @@ int orte_rmaps_rr_byobj(orte_job_t *jdata,
                         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
                         return ORTE_ERR_NOT_FOUND;
                     }
+                    if (orte_rmaps_base.cpus_per_rank > (int)opal_hwloc_base_get_npus(node->topology, obj)) {
+                        orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low", true,
+                                       orte_rmaps_base.cpus_per_rank, opal_hwloc_base_get_npus(node->topology, obj),
+                                       orte_rmaps_base_print_mapping(orte_rmaps_base.mapping));
+                        return ORTE_ERR_SILENT;
+                    }
                     if (NULL == (proc = orte_rmaps_base_setup_proc(jdata, node, app->idx))) {
                         return ORTE_ERR_OUT_OF_RESOURCE;
                     }
@@ -745,6 +751,12 @@ static int byobj_span(orte_job_t *jdata,
                 ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
                 return ORTE_ERR_NOT_FOUND;
             }
+            if (orte_rmaps_base.cpus_per_rank > (int)opal_hwloc_base_get_npus(node->topology, obj)) {
+                orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low", true,
+                               orte_rmaps_base.cpus_per_rank, opal_hwloc_base_get_npus(node->topology, obj),
+                               orte_rmaps_base_print_mapping(orte_rmaps_base.mapping));
+                return ORTE_ERR_SILENT;
+            }
             /* determine how many to map */
             nprocs = navg;
             if (0 < nxtra_objs) {
