@@ -712,12 +712,23 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_max_vm_size);
 
-    orte_set_slots = NULL;
+#if OPAL_HAVE_HWLOC
+    if (opal_hwloc_use_hwthreads_as_cpus) {
+        orte_set_slots = strdup("hwthreads");
+    } else {
+        orte_set_slots = strdup("cores");
+    }
     (void) mca_base_var_register ("orte", "orte", NULL, "set_default_slots",
-                                  "Set the number of slots on nodes that lack such info to the number of specified objects [a number, \"cores\", \"numas\", \"sockets\", or \"hwthreads\"]",
+                                  "Set the number of slots on nodes that lack such info to the"
+                                  " number of specified objects [a number, \"cores\" (default),"
+                                  " \"numas\", \"sockets\", \"hwthreads\" (default if hwthreads_as_cpus is set),"
+                                  " or \"none\" to skip this option]",
                                   MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_set_slots);
+#else
+    orte_set_slots = NULL;
+#endif
 
     /* should we display the allocation after determining it? */
     orte_display_allocation = false;
@@ -753,5 +764,14 @@ int orte_register_params(void)
                                   OPAL_INFO_LVL_9, MCA_BASE_VAR_SCOPE_READONLY,
                                   &orte_soft_locations);
 
+    /* allow specification of the cores to be used by daemons */
+    orte_daemon_cores = NULL;
+    (void) mca_base_var_register ("orte", "orte", NULL, "daemon_cores",
+                                  "Restrict the ORTE daemons (including mpirun) to operate on the specified cores (comma-separated list of ranges)",
+                                  MCA_BASE_VAR_TYPE_STRING, NULL, 0, 0,
+                                  OPAL_INFO_LVL_5, MCA_BASE_VAR_SCOPE_READONLY,
+                                  &orte_daemon_cores);
+
+
     return ORTE_SUCCESS;
 }
